@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2020 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -59,7 +58,7 @@ def _create_test_value(spec, dtype=None):
   if _is_numeric_type(spec.dtype):
     value = tensor_spec_utils.bounds(spec).min
   else:
-    value = tensor_utils.data_type_to_np_type(spec.dtype)()
+    value = tensor_utils.data_type_to_np_type(spec.dtype).type()
   shape = np.asarray(spec.shape)
   shape[shape < 0] = 1
   return np.full(shape=shape, fill_value=value, dtype=dtype)
@@ -71,16 +70,6 @@ def _create_test_tensor(spec, dtype=None):
   return tensor_utils.pack_tensor(value)
 
 
-def _np_range_info(np_type):
-  """Returns type info for `np_type`, which includes min and max attributes."""
-  if issubclass(np_type, np.floating):
-    return np.finfo(np_type)
-  elif issubclass(np_type, np.integer):
-    return np.iinfo(np_type)
-  else:
-    raise ValueError('{} does not have range info.'.format(np_type))
-
-
 def _below_min(spec):
   """Generates values below spec's min.
 
@@ -95,11 +84,12 @@ def _below_min(spec):
     return
 
   np_type = tensor_utils.data_type_to_np_type(spec.dtype)
-  min_type_value = _np_range_info(np_type).min
+  min_type_value = tensor_spec_utils.np_range_info(np_type).min
   minimum = tensor_spec_utils.bounds(spec).min
 
   for index in np.ndindex(*spec.shape):
-    if min_type_value < minimum[index]:
+    min_index_value = minimum if np.isscalar(minimum) else minimum[index]
+    if min_type_value < min_index_value:
       value = _create_test_value(spec)
       value[index] = min_type_value
       yield value, index
@@ -119,11 +109,12 @@ def _above_max(spec):
     return
 
   np_type = tensor_utils.data_type_to_np_type(spec.dtype)
-  max_type_value = _np_range_info(np_type).max
+  max_type_value = tensor_spec_utils.np_range_info(np_type).max
   maximum = tensor_spec_utils.bounds(spec).max
 
   for index in np.ndindex(*spec.shape):
-    if max_type_value > maximum[index]:
+    max_index_value = maximum if np.isscalar(maximum) else maximum[index]
+    if max_type_value > max_index_value:
       value = _create_test_value(spec)
       value[index] = max_type_value
       yield value, index
@@ -142,7 +133,7 @@ def _find_scalar_within_bounds(spec):
       return None
   else:
     np_type = tensor_utils.data_type_to_np_type(spec.dtype)
-    return np_type()
+    return np_type.type()
 
 
 def _step_before_test(function):
@@ -207,7 +198,7 @@ class Step(absltest.TestCase, metaclass=abc.ABCMeta):
   ##############################################################################
   def test_no_observations_returned_if_not_requested(self):
     observations = self.step().observations
-    self.assertEqual({}, observations)
+    self.assertEmpty(observations)
 
   def test_requested_observations_are_returned(self):
     response = self.step(requested_observations=self.observation_uids)
