@@ -15,10 +15,10 @@
 """Tests for Connection."""
 
 import contextlib
+from unittest import mock
 
 from absl.testing import absltest
 import grpc
-import mock
 
 from google.protobuf import any_pb2
 from google.protobuf import struct_pb2
@@ -68,8 +68,8 @@ _REQUEST_RESPONSE_PAIRS = {
 }
 
 
-def _process(request_iterator, metadata):
-  del metadata
+def _process(request_iterator, **kwargs):
+  del kwargs
   for request in request_iterator:
     yield _REQUEST_RESPONSE_PAIRS.get(request.SerializeToString(), _TEST_ERROR)
 
