@@ -14,7 +14,8 @@
 # ============================================================================
 """An implementation of a dm_env environment using dm_env_rpc."""
 
-from typing import Any, Iterable, Mapping, NamedTuple, Optional, Sequence
+from typing import Any, Mapping, NamedTuple, Optional, Sequence
+
 import dm_env
 import immutabledict
 
@@ -26,6 +27,7 @@ from dm_env_rpc.v1 import error
 from dm_env_rpc.v1 import spec_manager
 from dm_env_rpc.v1 import tensor_utils
 
+
 # Default observation names for common RL concepts.  By default the dm_env
 # wrapper will use these for reward and discount if available, but this behavior
 # can be overridden.
@@ -35,14 +37,18 @@ DEFAULT_DISCOUNT_KEY = 'discount'
 # Default key separator, used in flattening/unflattening nested structures.
 DEFAULT_KEY_SEPARATOR = '.'
 
+# Format string for error message used in DmEnvAdaptor.Reset
+_RESET_ENVIRONMENT_ERROR = r'''Environment changed spec after reset.
+before: "{specs}"
+after: "{new_specs}"'''
+
 
 class DmEnvAdaptor(dm_env.Environment):
   """An implementation of dm_env using dm_env_rpc as the data protocol.
 
   Users can also optionally provide a mapping of objects to DmEnvAdaptor
-  attributes. This is to accomodate user-created protocol extensions that
+  attributes. This is to accommodate user-created protocol extensions that
   compliment the core protocol.
-
   """
 
   # Disable pytype attribute checking for dynamically created extension attrs.
@@ -50,23 +56,23 @@ class DmEnvAdaptor(dm_env.Environment):
 
   def __init__(
       self,
-      connection: dm_env_rpc_connection.Connection,
+      connection: dm_env_rpc_connection.ConnectionType,
       specs: dm_env_rpc_pb2.ActionObservationSpecs,
       requested_observations: Optional[Sequence[str]] = None,
       nested_tensors: bool = True,
-      extensions: Optional[Mapping[str, Any]] = immutabledict.immutabledict()):
+      extensions: Mapping[str, Any] = immutabledict.immutabledict()):
     """Initializes the environment with the provided dm_env_rpc connection.
 
     Args:
-      connection: An instance of Connection already connected to a dm_env_rpc
-        server and after a successful JoinWorldRequest has been sent.
+      connection: An instance of ConnectionType already connected to a
+        dm_env_rpc server and after a successful JoinWorldRequest has been sent.
       specs: A dm_env_rpc ActionObservationSpecs message for the environment.
       requested_observations: List of observation names to be requested from the
         environment when step is called. If None is specified then all
         observations will be requested.
       nested_tensors: Boolean to determine whether to flatten/unflatten tensors.
-      extensions: Optional mapping of extension instances to DmEnvAdaptor
-        attributes. Raises ValueError if attribute already exists.
+      extensions: Mapping of extension instances to DmEnvAdaptor attributes.
+        Raises ValueError if attribute already exists.
     """
     self._dm_env_rpc_specs = specs
     self._action_specs = spec_manager.SpecManager(specs.actions)
@@ -111,18 +117,19 @@ class DmEnvAdaptor(dm_env.Environment):
     # Not strictly necessary but it makes the unit tests deterministic.
     self._requested_observation_uids.sort()
 
-    if extensions is not None:
-      for extension_name, extension in extensions.items():
-        if hasattr(self, extension_name):
-          raise ValueError(
-              f'DmEnvAdaptor already has attribute "{extension_name}"!')
-        setattr(self, extension_name, extension)
+    self._extension_names = extensions.keys()
+    for extension_name, extension in extensions.items():
+      if hasattr(self, extension_name):
+        raise ValueError(
+            f'DmEnvAdaptor already has attribute "{extension_name}"!')
+      setattr(self, extension_name, extension)
 
   def reset(self):
     """Implements dm_env.Environment.reset."""
-    response = self._connection.send(dm_env_rpc_pb2.ResetRequest())
-    if self._dm_env_rpc_specs != response.specs:
-      raise RuntimeError('Environment changed spec after reset')
+    reset_response = self._connection.send(dm_env_rpc_pb2.ResetRequest())
+    if self._dm_env_rpc_specs != reset_response.specs:
+      raise RuntimeError(_RESET_ENVIRONMENT_ERROR.format(
+          specs=self._dm_env_rpc_specs, new_specs=reset_response.specs))
     self._last_state = dm_env_rpc_pb2.EnvironmentStateType.INTERRUPTED
     return self.step({})
 
@@ -145,8 +152,12 @@ class DmEnvAdaptor(dm_env.Environment):
     elif self._last_state == dm_env_rpc_pb2.EnvironmentStateType.RUNNING:
       step_type = dm_env.StepType.LAST
     else:
+      # Neither response.state nor _last_state is RUNNING.
+      # See common causes for state transition errors:
+      # https://github.com/deepmind/dm_env_rpc/blob/master/docs/v1/appendix.md#common-state-transition-errors
       raise RuntimeError('Environment transitioned from {} to {}'.format(
-          self._last_state, step_response.state))
+          dm_env_rpc_pb2.EnvironmentStateType.Name(self._last_state),
+          dm_env_rpc_pb2.EnvironmentStateType.Name(step_response.state)))
 
     self._last_state = step_response.state
 
@@ -254,24 +265,32 @@ class DmEnvAdaptor(dm_env.Environment):
 
   def close(self):
     """Implements dm_env.Environment.close."""
+    # Release any extensions associated with this EnvAdaptor:
+    for extension_name in self._extension_names:
+      setattr(self, extension_name, None)
     # Leaves the world if we were joined.  If not, this will be a no-op anyway.
-    self._connection.send(dm_env_rpc_pb2.LeaveWorldRequest())
-    self._connection = None
+    if self._connection is not None:
+      self._connection.send(dm_env_rpc_pb2.LeaveWorldRequest())
+      self._connection = None
 
 
-def create_world(connection: dm_env_rpc_connection.Connection,
+def create_world(connection: dm_env_rpc_connection.ConnectionType,
                  create_world_settings: Mapping[str, Any]) -> str:
   """Helper function to create a world with the provided settings.
 
   Args:
     connection: An instance of Connection already connected to a dm_env_rpc
       server.
-    create_world_settings: Settings used to create the world. Values must be
-      packable into a Tensor proto or already packed.
+    create_world_settings: Settings used to create the world. Nested settings
+      will be automatically flattened before sending to the server. Values must
+      be packable into a Tensor proto or already packed.
 
   Returns:
     Created world name.
   """
+  create_world_settings = dm_env_flatten_utils.flatten_dict(
+      create_world_settings, DEFAULT_KEY_SEPARATOR
+  )
 
   create_world_settings = {
       key: (value if isinstance(value, dm_env_rpc_pb2.Tensor) else
@@ -284,12 +303,10 @@ def create_world(connection: dm_env_rpc_connection.Connection,
 
 
 def join_world(
-    connection: dm_env_rpc_connection.Connection,
+    connection: dm_env_rpc_connection.ConnectionType,
     world_name: str,
     join_world_settings: Mapping[str, Any],
-    requested_observations: Optional[Iterable[str]] = None,
-    extensions: Optional[Mapping[str, Any]] = immutabledict.immutabledict(),
-    nested_tensors: bool = True
+    **adaptor_kwargs,
 ) -> DmEnvAdaptor:
   """Helper function to join a world with the provided settings.
 
@@ -297,16 +314,18 @@ def join_world(
     connection: An instance of Connection already connected to a dm_env_rpc
       server.
     world_name: Name of the world to join.
-    join_world_settings: Settings used to join the world. Values must be
+    join_world_settings: Settings used to join the world. Nested settings will
+      be automatically flattened before sending to the server. Values must be
       packable into a Tensor message or already packed.
-    requested_observations: Optional set of requested observations.
-    extensions: Optional mapping of extension instances to DmEnvAdaptor
-      attributes.
-    nested_tensors: Boolean to determine whether to flatten/unflatten tensors.
+    **adaptor_kwargs: Additional keyword args used to create the DmEnvAdaptor
+      instance.
 
   Returns:
     Instance of DmEnvAdaptor.
   """
+  join_world_settings = dm_env_flatten_utils.flatten_dict(
+      join_world_settings, DEFAULT_KEY_SEPARATOR
+  )
 
   join_world_settings = {
       key: (value if isinstance(value, dm_env_rpc_pb2.Tensor) else
@@ -318,8 +337,7 @@ def join_world(
           world_name=world_name, settings=join_world_settings)).specs
 
   try:
-    return DmEnvAdaptor(
-        connection, specs, requested_observations, nested_tensors, extensions)
+    return DmEnvAdaptor(connection, specs, **adaptor_kwargs)
   except ValueError:
     connection.send(dm_env_rpc_pb2.LeaveWorldRequest())
     raise
@@ -331,13 +349,10 @@ class DmEnvAndWorldName(NamedTuple):
   world_name: str
 
 
-def create_and_join_world(
-    connection: dm_env_rpc_connection.Connection,
-    create_world_settings: Mapping[str, Any],
-    join_world_settings: Mapping[str, Any],
-    requested_observations: Optional[Iterable[str]] = None,
-    extensions: Optional[Mapping[str, Any]] = immutabledict.immutabledict(),
-    nested_tensors: bool = True) -> DmEnvAndWorldName:
+def create_and_join_world(connection: dm_env_rpc_connection.ConnectionType,
+                          create_world_settings: Mapping[str, Any],
+                          join_world_settings: Mapping[str, Any],
+                          **adaptor_kwargs) -> DmEnvAndWorldName:
   """Helper function to create and join a world with the provided settings.
 
   Args:
@@ -345,12 +360,11 @@ def create_and_join_world(
       server.
     create_world_settings: Settings used to create the world. Values must be
       packable into a Tensor proto or already packed.
-    join_world_settings: Settings used to join the world. Values must be
+    join_world_settings: Settings used to join the world. Nested settings will
+      be automatically flattened before sending to the server. Values must be
       packable into a Tensor message.
-    requested_observations: Optional set of requested observations.
-    extensions: Optional mapping of extension instances to DmEnvAdaptor
-      attributes.
-    nested_tensors: Boolean to determine whether to flatten/unflatten tensors.
+    **adaptor_kwargs: Additional keyword args used to create the DmEnvAdaptor
+      instance.
 
   Returns:
     Tuple of DmEnvAdaptor and the created world name.
@@ -359,8 +373,7 @@ def create_and_join_world(
   try:
     return DmEnvAndWorldName(
         join_world(connection, world_name, join_world_settings,
-                   requested_observations, extensions, nested_tensors),
-        world_name)
+                   **adaptor_kwargs), world_name)
   except (error.DmEnvRpcError, ValueError):
     connection.send(dm_env_rpc_pb2.DestroyWorldRequest(world_name=world_name))
     raise
