@@ -14,10 +14,11 @@
 # ============================================================================
 """Tests for dm_env_rpc/dm_env adaptor."""
 
+from unittest import mock
+
 from absl.testing import absltest
 import dm_env
 from dm_env import specs
-import mock
 import numpy as np
 
 from google.rpc import status_pb2
@@ -107,6 +108,38 @@ _RESERVED_STEP_RESPONSE = dm_env_rpc_pb2.StepResponse(
         1: tensor_utils.pack_tensor(5, dtype=dm_env_rpc_pb2.UINT8),
         2: tensor_utils.pack_tensor('goodbye')
     })
+_RESET_CHANGES_SPEC_ERROR = r'''Environment changed spec after reset.
+before: "actions {
+  key: 1
+  value {
+    name: "foo"
+    dtype: UINT8
+  }
+}
+actions {
+  key: 2
+  value {
+    name: "bar"
+    dtype: STRING
+  }
+}
+observations {
+  key: 1
+  value {
+    name: "foo"
+    dtype: UINT8
+  }
+}
+observations {
+  key: 2
+  value {
+    name: "bar"
+    dtype: STRING
+  }
+}
+"
+after: ""'''
+_EXTENSIONS = {'baz': 123, 'qux': 'quux'}
 
 
 class DmEnvAdaptorTests(absltest.TestCase):
@@ -114,7 +147,8 @@ class DmEnvAdaptorTests(absltest.TestCase):
   def setUp(self):
     super(DmEnvAdaptorTests, self).setUp()
     self._connection = mock.MagicMock()
-    self._env = dm_env_adaptor.DmEnvAdaptor(self._connection, _SAMPLE_SPEC)
+    self._env = dm_env_adaptor.DmEnvAdaptor(
+        self._connection, _SAMPLE_SPEC, extensions=_EXTENSIONS)
 
   def test_requested_observations(self):
     requested_observations = ['foo']
@@ -185,7 +219,8 @@ class DmEnvAdaptorTests(absltest.TestCase):
   def test_illegal_state_transition(self):
     self._connection.send = mock.MagicMock(
         return_value=_TERMINATED_STEP_RESPONSE)
-    with self.assertRaisesRegex(RuntimeError, 'Environment transitioned'):
+    with self.assertRaisesRegex(
+        RuntimeError, 'Environment transitioned from TERMINATED to TERMINATED'):
       self._env.step({})
 
   def test_reset(self):
@@ -213,7 +248,7 @@ class DmEnvAdaptorTests(absltest.TestCase):
             requested_observations=[1, 2],
             actions={
                 1: tensor_utils.pack_tensor(actions['foo']),
-                2: tensor_utils.pack_tensor(actions['bar'], dtype=np.str_)
+                2: tensor_utils.pack_tensor(actions['bar'], dtype=str)  # pytype: disable=wrong-arg-types  # typed-numpy
             }))
 
   def test_reset_changes_spec_raises_error(self):
@@ -222,7 +257,8 @@ class DmEnvAdaptorTests(absltest.TestCase):
     self._connection.send.assert_called_once_with(_SAMPLE_STEP_REQUEST)
     self._connection.send = mock.MagicMock(
         side_effect=[_RESET_CHANGES_SPEC_RESPONSE, _SAMPLE_STEP_RESPONSE])
-    with self.assertRaisesRegex(RuntimeError, 'changed spec'):
+    with self.assertRaisesWithLiteralMatch(RuntimeError,
+                                           _RESET_CHANGES_SPEC_ERROR):
       self._env.reset()
 
   def test_observation_spec(self):
@@ -268,6 +304,31 @@ class DmEnvAdaptorTests(absltest.TestCase):
     with self.assertRaisesRegex(ValueError, 'foo'):
       self._env.close()
 
+  def test_close_releases_extensions(self):
+    for extension_name, extension in _EXTENSIONS.items():
+      self.assertEqual(getattr(self._env, extension_name), extension)
+    self._connection.send = mock.MagicMock(
+        return_value=dm_env_rpc_pb2.LeaveWorldResponse())
+    self._env.close()
+    for extension_name in _EXTENSIONS:
+      self.assertIsNone(getattr(self._env, extension_name))
+
+  def test_close_allows_multiple_close_calls(self):
+    self._connection.send = mock.MagicMock(
+        return_value=dm_env_rpc_pb2.LeaveWorldResponse(),
+    )
+    self._env.close()
+    self._env.close()
+
+    # Most importantly, when calling close multiple times, we need to make sure
+    # that any _connection.send request is called once as the conenction is
+    # destroyed after the first close.
+
+    self._connection.send.assert_called_once_with(
+        dm_env_rpc_pb2.LeaveWorldRequest(),
+    )
+    self.assertIsNone(self._env._connection)
+
 
 class OverrideRewardDiscount(dm_env_adaptor.DmEnvAdaptor):
 
@@ -401,13 +462,13 @@ class EnvironmentNestedActionsObservations(absltest.TestCase):
         'foo': {
             'bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
         },
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
     expected_observations = {
         'foo': {
             'bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
         },
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
 
     self.assertSameElements(expected_actions, env.action_spec())
@@ -420,11 +481,11 @@ class EnvironmentNestedActionsObservations(absltest.TestCase):
         nested_tensors=False)
     expected_actions = {
         'foo.bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
     expected_observations = {
         'foo.bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
 
     self.assertSameElements(expected_actions, env.action_spec())
@@ -554,8 +615,10 @@ class CreateJoinHelpers(absltest.TestCase):
         join_world_settings={
             'ship_type': 1,
             'player': 'zaphod',
-        })
+        },
+        requested_observations=['foo'])
     self.assertIsNotNone(env)
+    self.assertSameElements(env.observation_spec().keys(), ['foo'])
     self.assertEqual('Damogran_01', world_name)
 
     connection.send.assert_has_calls([
@@ -573,6 +636,38 @@ class CreateJoinHelpers(absltest.TestCase):
                 }""", dm_env_rpc_pb2.JoinWorldRequest())),
     ])
 
+  def test_flatten_create_join_world_settings(self):
+    connection = mock.MagicMock()
+    connection.send = mock.MagicMock(side_effect=[
+        dm_env_rpc_pb2.CreateWorldResponse(world_name='Damogran_01'),
+        dm_env_rpc_pb2.JoinWorldResponse(specs=_SAMPLE_SPEC)
+    ])
+    env, world_name = dm_env_adaptor.create_and_join_world(
+        connection,
+        create_world_settings={'nested': {'planet': 'Damogran'}},
+        join_world_settings={'nested': {'ship_type': 1, 'player': 'zaphod'}})
+    self.assertIsNotNone(env)
+    self.assertEqual('Damogran_01', world_name)
+
+    connection.send.assert_has_calls([
+        mock.call(
+            text_format.Parse(
+                """settings: {
+                key: 'nested.planet', value: { strings: { array: 'Damogran' } }
+            }""", dm_env_rpc_pb2.CreateWorldRequest())),
+        mock.call(
+            text_format.Parse(
+                """world_name: 'Damogran_01'
+                settings: {
+                  key: 'nested.ship_type', value: { int64s: { array: 1 } }
+                }
+                settings: {
+                    key: 'nested.player', value: {
+                        strings: { array: 'zaphod' }
+                    }
+                }""", dm_env_rpc_pb2.JoinWorldRequest())),
+    ])
+
   def test_create_join_world_with_packed_settings(self):
     connection = mock.MagicMock()
     connection.send = mock.MagicMock(side_effect=[
@@ -644,11 +739,11 @@ class CreateJoinHelpers(absltest.TestCase):
         nested_tensors=False)
     expected_actions = {
         'foo.bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
     expected_observations = {
         'foo.bar': specs.Array(shape=(), dtype=np.int32, name='foo.bar'),
-        'baz': specs.Array(shape=(), dtype=np.str_, name='baz'),
+        'baz': specs.Array(shape=(), dtype=str, name='baz'),
     }
 
     self.assertSameElements(expected_actions, env.action_spec())
