@@ -15,10 +15,10 @@
 """Tests for dm_env_rpc helper functions."""
 
 import struct
+from unittest import mock
 
 from absl.testing import absltest
 from absl.testing import parameterized
-import mock
 import numpy as np
 
 from google.protobuf import any_pb2
@@ -84,9 +84,15 @@ class PackTensorTests(parameterized.TestCase):
       (25, np.uint32, 'uint32s'),
       (25, np.uint64, 'uint64s'),
       (2**64-1, np.uint64, 'uint64s'),
-      (True, np.bool, 'bools'),
-      (False, np.bool, 'bools'),
-      ('foo', np.str, 'strings'),
+      (True, bool, 'bools'),
+      (False, bool, 'bools'),
+      ('foo', str, 'strings'),
+      # Test legacy support for numpy built-in types (no longer recommended as
+      # of release 1.20.0 -
+      # https://numpy.org/devdocs/release/1.20.0-notes.html#deprecations)
+      (True, np.bool_, 'bools'),
+      (False, np.bool_, 'bools'),
+      ('foo', np.str_, 'strings'),
   )
   def test_pack_scalars_specific_dtype(self, scalar, dtype, expected_payload):
     tensor = tensor_utils.pack_tensor(scalar, dtype)
@@ -175,14 +181,14 @@ class PackTensorTests(parameterized.TestCase):
       (['foobar'], np.array(['foobar']).dtype),
   )
   def test_np_object_strings(self, value, dtype):
-    object_array = np.array(value, dtype=np.object)
+    object_array = np.array(value, dtype=object)
     tensor = tensor_utils.pack_tensor(object_array, dtype=dtype)
     self.assertEqual(list(object_array.shape), tensor.shape)
     self.assertTrue(tensor.HasField('strings'))
 
   def test_np_object_strings_no_dtype_raises_exception(self):
     with self.assertRaises(ValueError):
-      tensor_utils.pack_tensor(np.array(['foo'], dtype=np.object))
+      tensor_utils.pack_tensor(np.array(['foo'], dtype=object))
 
   @parameterized.parameters(
       (['foo', 42, 'bar'],),
@@ -192,7 +198,7 @@ class PackTensorTests(parameterized.TestCase):
     with self.assertRaisesRegex(TypeError,
                                 'not all elements are Python string types'):
       tensor_utils.pack_tensor(
-          np.array(bad_element, dtype=np.object), dtype=np.str_)
+          np.array(bad_element, dtype=object), dtype=np.str_)
 
   def test_class_instance_throw_exception(self):
 
@@ -425,7 +431,7 @@ class DataTypeToNpTypeTests(absltest.TestCase):
         tensor_utils.data_type_to_np_type(dm_env_rpc_pb2.DataType.FLOAT))
 
   def test_empty_object_list(self):
-    tensor = tensor_utils.pack_tensor(np.array([], dtype=np.object))
+    tensor = tensor_utils.pack_tensor(np.array([], dtype=object))
     self.assertEqual([0], tensor.shape)
 
   def test_unknown_type(self):
